function [g, fit_param] = calc_gr_tornado(t, od, t_init, t_min_fit, max_t_pass_bg, disp_output)

if ~exist('disp_output', 'var')
    disp_output = false;
end

i_1 = find(t>t_init,1);
i_f = find(t>t_min_fit,1);

init_od = sort(od(1:25), 'ascend');
s_bg = median(init_od(2:5));
s_bg = init_od(1);
max_od = 0.2*(max(od)-s_bg);

t01 = find(od-s_bg > max_od & (1:numel(od)>i_1)', 1);
if isempty(t01)
    t01 = 1000;
end
min_fit = min(i_f, t01);

j = 1;
n_it = [numel(t)-i_1-min_fit+1,1];

% prealocate variables;
pred = cell(n_it); ind = cell(n_it);
val = zeros(n_it); e = zeros(n_it);% h = zeros(n_it);

% looping on number of points in the fit;
for k = i_1+min_fit:numel(t)
    [ind{j}, pred{j}, val(j), e(j)] = F_calc_gr(t, od, i_1:k, max_t_pass_bg, s_bg);
    j = j+1;
end

% minimizing the error
rel_ind = find(val>=(cummax(val-e)) & val<=cummin(val+e));
[~, me] = min(e(rel_ind));
min_ind = rel_ind(me);

% preparing the data output

g = val(min_ind);
fit_param.g = g;
fit_param.bg = pred{min_ind}(3);
fit_param.n0 = pred{min_ind}(1);
fit_param.ind = i_1:i_1+min_fit-1+rel_ind(me);
fit_param.err = e(min_ind);



if disp_output
    disp_tornado_analysis(t, od, fit_param, g, val, e, i_1, min_fit);
end
end

function [ind, pred, val, e] = F_calc_gr(t, od, ind, max_t_pass_bg, s_bg)

l_bg = s_bg-2e-3;
h_bg = max(min(min(od(t>max_t_pass_bg))- 1e-4, s_bg*1.2), s_bg*1.1);

if sum(od(20:end)>s_bg) < 20
    ind = 0; pred = [0 0 s_bg - 3e-4]; val = 0; e = 0;
    return;
end

opt = optimoptions('lsqnonlin','Display','off');
x = t(ind); y = od(ind);
fun = @(a)sqrt((a(1)*2.^(a(2)*x)+a(3)-y).^2./(0.1+(a(1)*2.^(a(2)*x))).^2);
% fit the data using lsqnonlin
[pred, ~ ,residuals ,~,~,~,JAC] = lsqnonlin(fun, [1e-4 0.01 s_bg], [1e-6 0 l_bg], [1e-3 0.05 h_bg], opt);

if isempty(residuals)
    ind = 0; pred = [0 0 s_bg - 3e-4]; val = 0; e = 0;
    return;    
end

% calculate confidence intervals;
ci = nlparci(pred,residuals,'jacobian', JAC);
val = pred(2);
e = 2*(val-ci(2,1));
end

function disp_tornado_analysis(t, od, fit_param, g, val, e, i_1, min_fit)
scrz = get(0, 'ScreenSize');
fh1 = figure('Position', [scrz(3)*0.3 scrz(4)*0.3 scrz(3)*0.25 scrz(4)*0.35]);
sh(1) = axes('position', [0.125 0.525 0.85 0.46]);
semilogy(t(fit_param.ind), fit_param.n0*2.^(g*t(fit_param.ind)), '-r', 'linewidth', 2);
hold on;
semilogy(t, od-fit_param.bg, '.k');
xl = xlim;
xlim([0 2000]);
set(sh(1), 'xtick', [], 'fontsize', 10, 'ytick', [1e-4  1e-2 1e0], ...
    'YMinorTick', 'off', 'box', 'off');
ylabel('OD');

sh(2) = axes('position', [0.125 0.125 0.85 0.38]);
errorbar(t(i_1+min_fit:numel(t)), val', e');
xlim([0 2000]);
hold on; plot(t(fit_param.ind(end)), g, 'or');
set(sh(2), 'xtick', [0 500 1000 1500 2000], 'box', 'off');
ylim([0 0.033]);
linkaxes(sh, 'x');
ylabel('Growth rate ( min^{-1} )');
xlabel('Time ( min )');
end

